@@ -6,7 +6,7 @@ use Moo;
 use Carp;
 use Log::Any '$log';
 
-our $VERSION= '0.00_01';
+our $VERSION= '0.00_04';
 
 # ABSTRACT - Do What I Mean, with OpenGL on X11
 
@@ -220,7 +220,7 @@ sub create_render_pixmap {
 	my %args= @_ == 1 && ref($_[0]) eq 'HASH'? %{ $_[0] } : @_;
 	$args{width} && $args{height}
 		or croak "require 'width' and 'height'";
-	$args{depth} ||= $self->screen->depth;
+	$args{depth} ||= $self->visual_info->depth;
 	$log->tracef("create X pixmap: %s", \%args);
 	my $x_pixmap= $self->display->new_pixmap($self->screen, $args{width}, $args{height}, $args{depth});
 	$log->tracef("create GLX pixmap: %s %s", $self->visual_info, $x_pixmap) if $log->is_trace;
@@ -261,11 +261,11 @@ sub swap_buffers {
 
 
 my %_gl_err_msg= (
-	map { eval { OpenGL->$_() => $_ } } qw(
+	map { eval { OpenGL->can($_)->() => $_ } } qw(
 		GL_INVALID_ENUM
 		GL_INVALID_VALUE
 		GL_INVALID_OPERATION
-		GL_INVALIDFRAMEBUFFER_OPERATION
+		GL_INVALID_FRAMEBUFFER_OPERATION
 		GL_OUT_OF_MEMORY
 		GL_STACK_OVERFLOW
 		GL_STACK_UNDERFLOW
@@ -319,16 +319,28 @@ sub apply_gl_projection {
 	($t, $b)= ($b, $t) if $mirror_y;
 	$near= 1 unless defined $near;
 	$far= 1000 unless defined $far;
+	defined $_ or $_= 0
+		for ($x, $y, $z);
 	
-	$log->tracef('Setting projection matrix: l=%.4lf r=%.4lf b=%.4lf t=%.4lf near=%.4lf far=%.4lf',
-		$l, $r, $b, $t, $near, $far);
+	# If Z is specified, then the left/right/top/bottom are interpreted to be the
+	# edges of the screen at this position.  Only matters for Frustum.
+	if ($z && !$ortho) {
+		my $scale= 1.0/$z;
+		$l *= $scale;
+		$r *= $scale;
+		$t *= $scale;
+		$b *= $scale;
+	}
+	
+	$log->tracef('Setting projection matrix: l=%.4lf r=%.4lf b=%.4lf t=%.4lf near=%.4lf far=%.4lf; translate %.4lf,%.4lf,%.4lf',
+		$l, $r, $b, $t, $near, $far, -$x, -$y, -$z);
 	OpenGL::glMatrixMode(OpenGL::GL_PROJECTION());
 	OpenGL::glLoadIdentity();
 	
 	$ortho? OpenGL::glOrtho($l, $r, $b, $t, $near, $far)
 	      : OpenGL::glFrustum($l, $r, $b, $t, $near, $far);
 	
-	OpenGL::glTranslated(-($x||0), -($y||0), -($z||0))
+	OpenGL::glTranslated(-$x, -$y, -$z)
 		if $x or $y or $z;
 	
 	# If mirror is in effect, need to tell OpenGL which way the camera is
@@ -362,7 +374,7 @@ X11::GLX::DWIM
 
 =head1 VERSION
 
-version 0.00_01
+version 0.00_04
 
 =head1 SYNOPSIS
 
@@ -372,10 +384,13 @@ version 0.00_01
     my_custom_opengl_rendering();
     $glx->end_frame();
   }
+  
   # defaults above:
   #   Connect to default X11 Display
   #   32-bit RGBA visual, double buffered
   #   rendering to full-screen window.  Direct-render if supported.
+  #   begin_frame calls glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT)
+  #   end_frame calls glXSwapBuffers and reports any glError
 
 =head1 DESCRIPTION
 
@@ -410,8 +425,8 @@ Read-only, lazy-built from L</display>.
 
 Lazy-built, read-only.  Instance of L<X11::Xlib::XVisualInfo>.
 Can be initialized with an arrayref of parameters (integer codes) to pass to
-C<glXChooseVisual>, or with a hashref of fields to pass to the constructor of
-C<XVisualInfo>.
+L<glXChooseVisual|X11::GLX/glXChooseVisual>, or with a hashref of fields to
+pass to the constructor of C<XVisualInfo>.
 
 =head2 colormap
 
@@ -421,7 +436,7 @@ colormap compatible with L</visual_info>.
 =head2 glx_context
 
 An instance of L<X11::GLX::Context>.  You can also initialize it with a hash
-of arguments for the call to glXCreateContext:
+of arguments for the call to L<glXCreateContext|X11::GLX/glXCreateContext>
 
   $glx->glx_context({
     direct => $bool,            # for direct rendering ("DRI")
@@ -435,6 +450,9 @@ X11 ID for a GLX context with:
 
   my $xid= $glx->glx_context->id
 
+and then use that ID for the C<shared> option when creating later GL contexts
+in other processes.  See L<X11::GLX/Shared GL Contexts>.
+
 =head2 has_glx_context
 
 Returns whether glx_context has been initialized.
@@ -486,8 +504,32 @@ See L</apply_gl_projection>
 
 =head2 create_render_window
 
+  $glx->target( $glx->create_render_window( \%args ) );
+
+Create a window suitable for use as an OpenGL rendering target.
+C<%args> can be:
+
+  x        - default 0
+  y        - default 0
+  width    - default to screen width
+  height   - default to screen height
+  class    - default to InputOutput
+  ...
+
+There are dozens of other parameters you can specify.
+See L<X11::Xlib::Display/new_window>.
+
 =head2 create_render_pixmap
 
+  $glx->target( $glx->create_render_pixmap( \%args ) );
+
+Create a pixmap suitable for use as an OpenGL rendering target.
+C<%args> can be:
+
+  width - required
+  height - required
+  depth  - defaults to depth of your visual
+
 =head2 begin_frame
 
 Convenience method; initializes rendering target if it wasn't already done,
@@ -514,14 +556,18 @@ projection matrix.
 
   $glx->apply_gl_projection(
     ortho => $bool,
-    left => ..., right => ..., top => ..., bottom => ..., near => ..., far => ...,
-    x => ..., y => ..., z => ...,
+    left => ..., right => ..., top => ..., bottom => ...,
+    near => ..., far => ..., z => ...,
     aspect => ..., mirror_x => $bool, mirror_y => $bool,
   );
 
 If C<ortho> is true, it calls C<glOrtho>, else C<glFrustum>.
 The C<left>, C<right>, C<top>, C<bottom>, C<near>, C<far> parameters are as
-documented for these functions.
+documented for these functions, B<but> if you request a Frustum and specify a
+non-zero C<z> then it scales the parameters so that a vertex C<($left,$top,$z)>
+displays at the upper-left corner of the screen.  (normally the upper left
+would be C<($left,$top,$near)>)  This allows you to separate the near clipping
+plane from the plane you use to scale your coordinates.
 
 If you specify C<x>, C<y>, or C<z>, it calls C<glTranslated(-x,-y,-z)> after
 the C<glFrustum> or C<glOrtho>.
@@ -535,7 +581,7 @@ coordinate system.  If both dimensions are missing, top defaults to C<-bottom>,
 or C<1>, and the rest is calculated from that.
 
 If you specify C<mirror_x> or C<mirror_y>, it will flip the coordinate system
-so that C<-x> is leftward or C<-y> is upward.  (remember, GL coordinates have
+so that C<+x> is leftward or C<+y> is downward.  (remember, GL coordinates have
 C<+y> upward by default).  This will also call C<glFrontFace> to match, so
 mirrored X or Y is C<GL_CW> (clockwise) and neither mirrored or both mirrored
 is the default C<GL_CCW> (counter clockwise).
